@@ -16,7 +16,7 @@ arbitrary tcp/ip connections, and the authentication agent connection.
 */
 
 #include "includes.h"
-RCSID("$Id: channels.c,v 1.15 1999/10/31 09:35:47 bg Exp $");
+RCSID("$Id: channels.c,v 1.16 1999/11/29 13:20:20 bg Exp $");
 
 #ifndef HAVE_GETHOSTNAME
 #include <sys/utsname.h>
@@ -204,7 +204,7 @@ void channel_prepare_select(fd_set *readset, fd_set *writeset)
 
 	case SSH_CHANNEL_OPEN:
 	  if(compat13){
-	  if (buffer_len(&ch->input) < 32768)
+	  if (buffer_len(&ch->input) < packet_get_maxsize())
 	    FD_SET(ch->sock, readset);
 	  if (buffer_len(&ch->output) > 0)
 	    FD_SET(ch->sock, writeset);
@@ -212,7 +212,7 @@ void channel_prepare_select(fd_set *readset, fd_set *writeset)
 	  }
 	  /* test whether sockets are 'alive' for read/write */
           if (ch->istate == CHAN_INPUT_OPEN)
-            if (buffer_len(&ch->input) < 32768)
+            if (buffer_len(&ch->input) < packet_get_maxsize())
               FD_SET(ch->sock, readset);
           if (ch->ostate == CHAN_OUTPUT_OPEN || ch->ostate == CHAN_OUTPUT_WAIT_DRAIN){
             if (buffer_len(&ch->output) > 0){
@@ -599,9 +599,9 @@ int channel_not_very_much_buffered_data()
 	case SSH_CHANNEL_AUTH_SOCKET:
 	  continue;
 	case SSH_CHANNEL_OPEN:
-	  if (buffer_len(&ch->input) > 32768)
+	  if (buffer_len(&ch->input) > packet_get_maxsize())
 	    return 0;
-	  if (buffer_len(&ch->output) > 32768)
+	  if (buffer_len(&ch->output) > packet_get_maxsize())
 	    return 0;
 	  continue;
 	case SSH_CHANNEL_INPUT_DRAINING:
