@@ -14,7 +14,7 @@ precision integers.
 
 */
 
-/* RCSID("$Id: mpaux.h,v 1.2 1999/05/04 11:58:52 bg Exp $"); */
+/* RCSID("$Id: mpaux.h,v 1.3 1999/10/31 12:31:29 bg Exp $"); */
 
 #ifndef MPAUX_H
 #define MPAUX_H
@@ -22,11 +22,11 @@ precision integers.
 /* Converts a multiple-precision integer into bytes to be stored in the buffer.
    The buffer will contain the value of the integer, msb first. */
 void mp_linearize_msb_first(unsigned char *buf, unsigned int len, 
-			    MP_INT *value);
+			    BIGNUM *value);
 
 /* Extract a multiple-precision integer from buffer.  The value is stored
    in the buffer msb first. */
-void mp_unlinearize_msb_first(MP_INT *value, const unsigned char *buf,
+void mp_unlinearize_msb_first(BIGNUM *value, const unsigned char *buf,
 			      unsigned int len);
 
 /* Computes a 16-byte session id in the global variable session_id.
@@ -35,8 +35,8 @@ void mp_unlinearize_msb_first(MP_INT *value, const unsigned char *buf,
 void compute_session_id(unsigned char session_id[16],
 			unsigned char cookie[8],
 			unsigned int host_key_bits,
-			MP_INT *host_key_n,
+			BIGNUM *host_key_n,
 			unsigned int session_key_bits,
-			MP_INT *session_key_n);
+			BIGNUM *session_key_n);
 
 #endif /* MPAUX_H */
