@@ -13,12 +13,11 @@ Generic header file for ssh.
 
 */
 
-/* RCSID("$Id: ssh.h,v 1.9 1999/06/06 08:35:30 bg Exp $"); */
+/* RCSID("$Id: ssh.h,v 1.20 1999/11/11 21:28:10 bg Exp $"); */
 
 #ifndef SSH_H
 #define SSH_H
 
-#include <gmp.h>
 #include "rsa.h"
 #include "randoms.h"
 #include "cipher.h"
@@ -47,7 +46,7 @@ Generic header file for ssh.
 
 /* Minor protocol version.  Different version indicates minor incompatibility
    that does not prevent interoperation. */
-#define PROTOCOL_MINOR		3
+#define PROTOCOL_MINOR		5
 
 /* Name for the service.  The port named by this service overrides the default
    port if present. */
@@ -129,6 +128,10 @@ only by root, whereas ssh_config should be world-readable. */
    socket is stored in /tmp, which is supposedly on the local machine.  If
    this were in the user\'s home directory, the daemon (running as root)
    might not be able to create and chown the file to the user\'s uid. */
+/*
+ * SSH_AGENT_SOCKET is now a directory that contains the socket
+ * /tmp/ssh_agent.%d/socket.
+ */
 #define SSH_AGENT_SOCKET	"/tmp/ssh_agent.%d"
 
 /* Name of the environment variable containing the authentication fd. */
@@ -136,7 +139,7 @@ only by root, whereas ssh_config should be world-readable. */
 
 /* Name of the environment variable containing the pathname of the
    authentication socket. */
-#define SSH_AUTHSOCKET_ENV_NAME	"SSH_AUTHENTICATION_SOCKET"
+#define SSH_AUTHSOCKET_ENV_NAME	"SSH_AUTH_SOCK"
 
 /* Force host key length and server key length to differ by at least this
    many bits.  This is to make double encryption with rsaref work. */
@@ -158,6 +161,7 @@ only by root, whereas ssh_config should be world-readable. */
 				/* 5 is TIS */
 #define SSH_AUTH_KERBEROS	6
 #define SSH_PASS_KERBEROS_TGT	7
+				/* 8 to 15 are reserved */
 #define SSH_PASS_AFS_TOKEN	21
 
 /* Protocol flags.  These are bit masks. */
@@ -172,12 +176,12 @@ only by root, whereas ssh_config should be world-readable. */
 #define SSH_MSG_NONE				0	/* no message */
 #define SSH_MSG_DISCONNECT			1	/* cause (string) */
 #define SSH_SMSG_PUBLIC_KEY			2	/* ck,msk,srvk,hostk */
-#define SSH_CMSG_SESSION_KEY			3	/* key (MP_INT) */
+#define SSH_CMSG_SESSION_KEY			3	/* key (BIGNUM) */
 #define SSH_CMSG_USER				4	/* user (string) */
 #define SSH_CMSG_AUTH_RHOSTS			5	/* user (string) */
-#define SSH_CMSG_AUTH_RSA			6	/* modulus (MP_INT) */
-#define SSH_SMSG_AUTH_RSA_CHALLENGE		7	/* int (MP_INT) */
-#define SSH_CMSG_AUTH_RSA_RESPONSE		8	/* int (MP_INT) */
+#define SSH_CMSG_AUTH_RSA			6	/* modulus (BIGNUM) */
+#define SSH_SMSG_AUTH_RSA_CHALLENGE		7	/* int (BIGNUM) */
+#define SSH_CMSG_AUTH_RSA_RESPONSE		8	/* int (BIGNUM) */
 #define SSH_CMSG_AUTH_PASSWORD			9	/* pass (string) */
 #define SSH_CMSG_REQUEST_PTY		        10	/* TERM, tty modes */
 #define SSH_CMSG_WINDOW_SIZE		        11	/* row,col,xpix,ypix */
@@ -207,6 +211,10 @@ only by root, whereas ssh_config should be world-readable. */
 #define SSH_CMSG_AUTH_RHOSTS_RSA		35	/* user,mod (s,mpi) */
 #define SSH_MSG_DEBUG				36	/* string */
 #define SSH_CMSG_REQUEST_COMPRESSION		37	/* level 1-9 (int) */
+#define SSH_CMSG_MAX_PACKET_SIZE		38	/* size 4k-1024k (int) */
+#define SSH_CMSG_AUTH_TIS			39	/* this is proto-1.5, but we ignore TIS */
+#define SSH_SMSG_AUTH_TIS_CHALLENGE		40
+#define SSH_CMSG_AUTH_TIS_RESPONSE		41
 
 #define SSH_CMSG_AUTH_KERBEROS			42	/* (KTEXT) */
 #define SSH_SMSG_AUTH_KERBEROS_RESPONSE		43	/* (KTEXT) */
@@ -255,7 +263,7 @@ int ssh_connect(const char *host, int port, int connection_attempts,
    If login fails, this function prints an error and never returns. 
    This initializes the random state, and leaves it initialized (it will also
    have references from the packet module). */
-void ssh_login(RandomState *state, int host_key_valid, RSAPrivateKey *host_key,
+void ssh_login(RandomState *state, int host_key_valid, RSA *host_key,
 	       const char *host, Options *options, uid_t original_real_uid);
 
 /*------------ Definitions for various authentication methods. -------*/
@@ -271,22 +279,22 @@ int auth_rhosts(struct passwd *pw, const char *client_user,
    its host key.  Returns true if authentication succeeds. */
 int auth_rhosts_rsa(RandomState *state,
 		    struct passwd *pw, const char *client_user,
-		    unsigned int bits, MP_INT *client_host_key_e,
-		    MP_INT *client_host_key_n, int ignore_rhosts,
+		    unsigned int bits, BIGNUM *client_host_key_e,
+		    BIGNUM *client_host_key_n, int ignore_rhosts,
 		    int strict_modes);
 
 /* Tries to authenticate the user using password.  Returns true if
    authentication succeeds. */
-int auth_password(const char *server_user, const char *password);
+int auth_password(struct passwd *pw, const char *password);
 
 /* Performs the RSA authentication dialog with the client.  This returns
    0 if the client could not be authenticated, and 1 if authentication was
    successful.  This may exit if there is a serious protocol violation. */
-int auth_rsa(struct passwd *pw, MP_INT *client_n, RandomState *state);
+int auth_rsa(struct passwd *pw, BIGNUM *client_n, RandomState *state, int strict_modes);
 
 /* Parses an RSA key (number of bits, e, n) from a string.  Moves the pointer
    over the key.  Skips any whitespace at the beginning and at end. */
-int auth_rsa_read_key(char **cpp, unsigned int *bitsp, MP_INT *e, MP_INT *n);
+int auth_rsa_read_key(char **cpp, unsigned int *bitsp, BIGNUM *e, BIGNUM *n);
 
 /* Returns the name of the machine at the other end of the socket.  The
    returned string should be freed by the caller. */
@@ -320,18 +328,18 @@ int match_hostname(const char *host, const char *pattern, unsigned int len);
 typedef enum { HOST_OK, HOST_NEW, HOST_CHANGED } HostStatus;
 HostStatus check_host_in_hostfile(const char *filename, 
 				  const char *host, unsigned int bits,
-				  MP_INT *e, MP_INT *n);
+				  BIGNUM *e, BIGNUM *n);
 
 /* Appends an entry to the host file.  Returns false if the entry
    could not be appended. */
 int add_host_to_hostfile(const char *filename, const char *host,
-			 unsigned int bits, MP_INT *e, MP_INT *n);
+			 unsigned int bits, BIGNUM *e, BIGNUM *n);
 
 /* Performs the RSA authentication challenge-response dialog with the client,
    and returns true (non-zero) if the client gave the correct answer to
    our challenge; returns zero if the client gives a wrong answer. */
 int auth_rsa_challenge_dialog(RandomState *state, unsigned int bits,
-			      MP_INT *e, MP_INT *n);
+			      BIGNUM *e, BIGNUM *n);
 
 /* Reads a passphrase from /dev/tty with echo turned off.  Returns the 
    passphrase (allocated with xmalloc).  Exits if EOF is encountered. 
@@ -343,7 +351,7 @@ char *read_passphrase(const char *prompt, int from_stdin);
    will precede the key to provide identification of the key without
    needing a passphrase. */
 int save_private_key(const char *filename, const char *passphrase,
-		     RSAPrivateKey *private_key, const char *comment,
+		     RSA *private_key, const char *comment,
 		     RandomState *state);
 
 /* Loads the public part of the key file (public key and comment). 
@@ -359,7 +367,7 @@ int load_public_key(const char *filename, RSAPublicKey *pub,
    in comment_return if it is non-NULL; the caller must free the value 
    with xfree. */
 int load_private_key(const char *filename, const char *passphrase,
-		     RSAPrivateKey *private_key, char **comment_return);
+		     RSA *private_key, char **comment_return);
 
 /*------------ Definitions for logging. -----------------------*/
 
@@ -390,6 +398,9 @@ void log_init(char *av0, int on_stderr, int debug, int quiet,
    characters.  The message should not contain newline. */
 void log(const char *fmt, ...);
 
+/* And syslog on LOG_AUTH. */
+void log_auth(const char *fmt, ...);
+
 /* Outputs a message to syslog or stderr, depending on the implementation. 
    The format must guarantee that the final message does not exceed 1024 
    characters.  The message should not contain newline. */
@@ -559,11 +570,6 @@ int match_pattern(const char *s, const char *pattern);
    Warning: this calls getpw*. */
 char *tilde_expand_filename(const char *filename, uid_t my_uid);
 
-/* Gets a file descriptor that won't get closed by shell pathname.
-   If pathname is NULL, the path is inferred from the SHELL environment
-   variable or the user id. */
-int get_permanent_fd(const char *pathname);
-
 /* Performs the interactive session.  This handles data transmission between
    the client and the program.  Note that the notion of stdin, stdout, and
    stderr in this function is sort of reversed: this function writes to
@@ -586,7 +592,7 @@ struct envstring {
 int ssh_tf_init(uid_t uid);
 int auth_krb4(const char *server_user, KTEXT auth, char **client);
 int auth_kerberos_tgt(struct passwd *pw, const char *string);
-int auth_afs_token(char *server_user, uid_t uid, const char *string);
+int auth_afs_token(struct passwd *pw, const char *token_string);
 
 int creds_to_radix(CREDENTIALS *creds, unsigned char *buf);
 int radix_to_creds(const char *buf, CREDENTIALS *creds);
